<?php
/*
Plugin Name: Measure Converter Widget
Description: Simple measure converter allowing to add and remove units and group of units at your own discretion. Custom units can also be added at user's own preference.
Version: 0.1
Author: Cabinetry Depot
Author URI: http://www.cabinetrydepot.com/
*/

class Measure_Converter_Widget extends WP_Widget {


  /** constructor */
  function Measure_Converter_Widget() {
    $widget_options = array('description' => 'Simple measure converter');
    parent::WP_Widget(false, $name = 'Measure_Converter_Widget');
  }

  function onActivate(){

    $measures = array(

      'Length' => array(

        'units' => array(

          'meter' => array(
            'scale' => 1
          ),

          'millimeter' => array(
            'scale' => 0.001
          ),

          'centimeter' => array(
            'scale' => 0.01
          ),

          'foot' => array(
            'scale' => 0.3048
          ),

          'inch' => array(
            'scale' => 0.0254
          ),

          'rod' => array(
            'scale' => 5.0292
          ),

          'yard' => array(
            'scale' => 0.9144
          ),

          'mile' => array(
            'scale' => 1609.344
          ),

          'nautical mile' => array(
            'scale' => 1852
          ),

          'kilometer' => array(
            'scale' => 1000
          )
        ),
        'active' => true
      ),

      'Angle' => array(

        'units' => array(

          'radian' => array(
            'scale' => 1
          ),

          'degree' => array(
            'scale' => M_PI/180
          ),

          'gradian' => array(
            'scale' => M_PI/200
          )
        ),
        'active' => true
      ),

      'Area' => array(

        'units' => array(

          'square meters' => array(
            'scale' => 1
          ),

          'square centimeter' => array(
            'scale' => 1e-4
          ),

          'square foot' => array(
            'scale' => 0.09290304
          ),

          'square inch' => array(
            'scale' => 0.00064516
          ),

          'square kilometer' => array(
            'scale' => 1e6
          ),

          'square mile' => array(
            'scale' => 2589988.11
          ),

          'square millimeter' => array(
            'scale' => 1e-6
          ),

          'square yard' => array(
            'scale' => 0.83612736
          ),

          'acres' => array(
            'scale' => 4046.85642
          ),

          'hectares' => array(
            'scale' => 1e4
          ),
        ),
        'active' => true
      ),

      'Temperature' => array(

        'units' => array(

          'celsius' => array(
            'scale' => 1,
            'shift' => 0
          ),

          'kelvin' => array(
            'scale' => 1,
            'shift' => -273.15
          ),

          'fahrenheit' => array(
            'scale' => (5/9),
            'shift' => -32
          ),
        ),
        'active' => true
      ),

      'Volume Fluid' => array(

        'units' => array(

          'liter' => array(
            'scale' => 1
          ),

          'milliliter' => array(
            'scale' => 0.001
          ),

          'fluid ounce UK' => array(
            'scale' => 0.0284130625
          ),

          'fluid ounce US' => array(
            'scale' => 0.0295735295625
          ),

          'gallon UK' => array(
            'scale' => 4.54609
          ),

          'gallon US' => array(
            'scale' => 3.785411784
          ),

          'pint UK' => array(
            'scale' => 0.56826125
          ),

          'pint US' => array(
            'scale' => 0.473176473
          ),

          'quart UK' => array(
            'scale' => 1.1365225
          ),
          'quart US' => array(
            'scale' => 0.946352946
          )
        ),
        'active' => true
      ),

      'Volume Solid' => array(

        'units' => array(

          'cubic meter' => array(
            'scale' => 1
          ),

          'cubic centimeter' => array(
            'scale' => 0.000001
          ),

          'cubic feet' => array(
            'scale' => 0.028316846592
          ),

          'cubic inch' => array(
            'scale' => 0.000016387064
          ),

          'cubic yard' => array(
            'scale' => 0.764554857984
          ),
        ),
        'active' => true
      )

    );

    foreach($measures as &$measure){
      foreach($measure['units'] as &$unit){
        $unit['active'] = true;
      }
    }

    if(!get_option('MeasureConverterWidgetData')){
      update_option('MeasureConverterWidgetData', $measures);
    }

  }

  function onDelete(){
    delete_option('MeasureConverterWidgetData');
  }


  /** @see WP_Widget::widget */
  function widget($args, $instance) {

    extract( $args );

    $title = apply_filters('widget_title', $instance['title']);
    $data = get_option('MeasureConverterWidgetData');
    $units = $data[key($data)]['units'];

    ?>

    <?php echo $before_widget; ?>
    <?php if($title) echo $before_title . $title . $after_title; ?>
    <div class="measureConverter">
      <div class="selectMeasure-wr">
        <select class="selectMeasure">
          <?php foreach($data as $measure_name => $measure_units): if($measure_units['active']):?>
          <option value="<?php echo $measure_name?>"><?php echo $measure_name?></option>
          <?php endif; endforeach; ?>
        </select>
      </div>
      <fieldset><legend><?php _e('From'); ?></legend>
        <select class="selectUnits from">
          <?php foreach($units as $unit_name => $unit): if($unit['active']):?>
          <option value="<?php echo $unit_name?>"><?php echo $unit_name?></option>
          <?php endif; endforeach;?>
        </select>
        <input type="text" class="convertedValue">
      </fieldset>
      <fieldset><legend><?php _e('To'); ?></legend>
        <select class="selectUnits to">
          <?php foreach($units as $unit_name => $unit): if($unit['active']):?>
          <option value="<?php echo $unit_name?>"><?php echo $unit_name?></option>
          <?php endif; endforeach;?>
        </select>
        <input type="text" class="convertedResult">
      </fieldset>
      <?php if(!$instance['hidePowered']):?>
        <div class="small">Provided by <a href="http://www.cabinetrydepot.com/">Cabinetry Depot</a></div>
      <?php endif;?>
    </div>
    <?php echo $after_widget; ?>

    <?php
    }

  /** @see WP_Widget::update */
  function update($new_instance, $old_instance) {

    $data = get_option('MeasureConverterWidgetData');
    $post = $_POST['data'];

    // Updating measures array
    foreach($data as $measureName => &$measure){
      $measure['active'] = isset($post[$measureName]['active']);
      foreach($measure['units'] as $unitName => &$unit){
        $unit['active'] = isset($post[$measureName]['units'][$unitName]['active']);
      }
    }

    // Adding new custom units to data array
    foreach($_POST['addNew'] as $group => $customUnits){
      foreach($customUnits as $customUnit){

        if(empty($customUnit['unitVal']) || empty($customUnit['unitName'])){
          continue;
        }

        $data[$group]['units'][ $customUnit['unitName'] ] = array(
          'scale'  => $customUnit['unitVal'],
          'active' => 1
        );
        
      }
    }

    update_option('MeasureConverterWidgetData',$data);

    $instance = $old_instance;
    $instance['title'] = strip_tags($new_instance['title']);
    $instance['hidePowered'] = isset($new_instance['hidePowered']);
    return $instance;
  }

  /** @see WP_Widget::form */
  function form($instance) {
    $title = esc_attr($instance['title']);
    $data = get_option('MeasureConverterWidgetData');
    ?>
    <p>
      <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" />
    </p>
    <div class="measureConverterOptions">
      <ul>
        <?php foreach($data as $measureName => $measure):?>
        <li class="measureNode" data-measure-name="<?php echo $measureName?>">
          <div><label><input type="checkbox" class="root" name="data[<?php echo $measureName?>][active]" <?php if($measure['active']) echo 'checked="checked"';?>> <?php echo $measureName?></label>
          <a href="#delete" class="deleteGroup" data-unit-group="<?php echo $measureName?>">delete</a></div>
          <ul <?php if(!$measure['active']) echo 'style="display:none;"'?>>
            <?php foreach($measure['units'] as $unitName => $unit):?>
            <li>
              <label><input type="checkbox" class="unit"  name="data[<?php echo $measureName?>][units][<?php echo $unitName?>][active]" <?php if($unit['active']) echo 'checked="checked"';?>> <?php echo $unitName?></label>
              <a href="#delete" class="deleteUnit" data-unit='{"name":"<?php echo $unitName?>", "group":"<?php echo $measureName?>"}'><?php _e('delete'); ?></a>
            </li>
            <?php endforeach;?>
            <li class="addCustomUnit">
              <p style="display:none;"><?php _e('Specify the name of new unit and the count of '); ?><b><?php echo key($measure['units']) ?></b><?php _e('(s) in your custom unit'); ?></p>
              <a href="#add"><?php _e('Add custom'); ?></a>
            </li>
          </ul>
        </li>
        <?php endforeach; ?>
        <li class="addCustomGroup">
          <p style="display:none;"><?php _e('Specify the name of new group and the main unit. All other units should be set depends on main unit.'); ?></p>
          <a href="#add"><?php _e('Add custom group'); ?></a>
        </li>
      </ul>
    <p>
      <input id="<?php echo $this->get_field_id('hidePowered'); ?>" name="<?php echo $this->get_field_name('hidePowered'); ?>" type="checkbox" <?php checked(isset($instance['hidePowered']) ? $instance['hidePowered'] : 0); ?> />&nbsp;
      <label for="<?php echo $this->get_field_id('hidePowered'); ?>"><?php _e('Hide "Provided by" link If hidden, please consider giving us link somewhere on your site'); ?></label>
    </p>
    </div>

    <?php
  	}

  function addJsAndCss(){
    wp_enqueue_script('measure-converter', plugins_url('/js/measure-converter.js', __FILE__),array('jquery'), false, true);
    wp_enqueue_style('measure-converter-styles', plugins_url('/css/converter_styles.css', __FILE__));
  }

  function getMeasurementUnits(){

    $type = isset($_GET['type']) ? $_GET['type'] : die('Error: Type parameter required!');

    $data = get_option('MeasureConverterWidgetData');

    if(!isset($data[$type])){
      die('Error: Type of measurement not found!');
    }

    $result = array();
    foreach($data[$type]['units'] as $unit_name => $unit){
      if($unit['active'])
        $result[] = $unit_name;
    }
    
    echo json_encode($result);
    exit;
  }

  function convert(){

    if(!isset($_POST['type']) || !isset($_POST['fromUnits']) || !isset($_POST['toUnits']) || !isset($_POST['value'])
       || empty($_POST['type']) || empty($_POST['fromUnits']) || empty($_POST['toUnits'])    )
    {
      die('Error: type, fromUnits, toUnits, value are required');
    }

    $type = $_POST['type'];
    $from = $_POST['fromUnits'];
    $to = $_POST['toUnits'];
    $value = (int)$_POST['value'];
    $data = get_option('MeasureConverterWidgetData');
    $units = $data[$type]['units'];

    if(!isset($units['shift'])){
      $units['shift'] = 0;
    }

    $result = (($value + $units[$from]['shift']) * $units[$from]['scale']) / $units[$to]['scale'] - $units[$to]['shift'];
    
    echo json_encode($result);
    exit;
  }

  function deleteMeasurementUnit(){

    if(!isset($_POST['unitName']) || !isset($_POST['unitGroup']))
      die('unitName and unitGroup are required');

    $uName = $_POST['unitName'];
    $uGroup = $_POST['unitGroup'];

    $data = get_option('MeasureConverterWidgetData');

    if(isset($data[$uGroup]['units'][$uName])){
      unset($data[$uGroup]['units'][$uName]);
    }

    update_option('MeasureConverterWidgetData',$data);

    echo 'ok';
    exit;
  }

  function deleteMeasurementUnitGroup(){

    if(!isset($_POST['unitGroup']))
      die('unitGroup is required');

    $uGroup = $_POST['unitGroup'];

    $data = get_option('MeasureConverterWidgetData');

    if(isset($data[$uGroup])){
      unset($data[$uGroup]);
    }

    update_option('MeasureConverterWidgetData',$data);

    echo 'ok';
    exit;
  }


} // class Measure_Converter_Widget

add_action('widgets_init', create_function('', 'return register_widget("Measure_Converter_Widget");'));
register_activation_hook( __FILE__, array('Measure_Converter_Widget', 'onActivate') );
register_uninstall_hook(__FILE__, array('Measure_Converter_Widget', 'onDelete'));
add_action('init', array('Measure_Converter_Widget', 'addJsAndCss'));
add_action('wp_ajax_getMeasurementUnits', array('Measure_Converter_Widget', 'getMeasurementUnits'));
add_action('wp_ajax_nopriv_getMeasurementUnits', array('Measure_Converter_Widget', 'getMeasurementUnits'));
add_action('wp_ajax_measurementConversion', array('Measure_Converter_Widget', 'convert'));
add_action('wp_ajax_nopriv_measurementConversion', array('Measure_Converter_Widget', 'convert'));
add_action('wp_ajax_deleteMeasurementUnit', array('Measure_Converter_Widget', 'deleteMeasurementUnit'));
add_action('wp_ajax_nopriv_deleteMeasurementUnit', array('Measure_Converter_Widget', 'deleteMeasurementUnit'));
add_action('wp_ajax_deleteMeasurementUnitGroup', array('Measure_Converter_Widget', 'deleteMeasurementUnitGroup'));
add_action('wp_ajax_nopriv_deleteMeasurementUnitGroup', array('Measure_Converter_Widget', 'deleteMeasurementUnitGroup'));